// Copyright (c) 2003-present, Jodd Team (http://jodd.org)
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
// 1. Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
//
// 2. Redistributions in binary form must reproduce the above copyright
// notice, this list of conditions and the following disclaimer in the
// documentation and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.

package jodd.decora;

import jodd.servlet.wrapper.BufferResponseWrapper;
import jodd.servlet.wrapper.LastModifiedData;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * Decora response wrapper uses {@link DecoraManager decora manager} to determine
 * if buffering should be enabled or disabled.
 */
public class DecoraResponseWrapper extends BufferResponseWrapper {

	protected final DecoraManager decoraManager;
	protected final HttpServletRequest request;
	protected final HttpServletResponse response;

	public DecoraResponseWrapper(final HttpServletRequest originalRequest, final HttpServletResponse originalResponse, final LastModifiedData lastModifiedData, final DecoraManager decoraManager) {
		super(originalResponse, lastModifiedData);
		this.request = originalRequest;
		this.response = originalResponse;
		this.decoraManager = decoraManager;

		// reset servlet response content length AFTER the chain, since
		// servlet container may set it and we are changing the content.
		originalResponse.setContentLength(-1);
	}

	@Override
	protected boolean bufferContentType(final String contentType, final String mimeType, final String encoding) {
		return decoraManager.decorateContentType(contentType, mimeType, encoding);
	}

	@Override
	protected boolean bufferStatusCode(final int statusCode) {
		return decoraManager.decorateStatusCode(statusCode);
	}

	// todo move to BufferResponseWrapper ?
	@Override
	protected void preResponseCommit() {
		long lastModified = lastModifiedData.getLastModified();
		long ifModifiedSince = request.getDateHeader("If-Modified-Since");

		if (lastModified > -1 && !response.containsHeader("Last-Modified")) {
			if (ifModifiedSince < (lastModified / 1000 * 1000)) {
				response.setDateHeader("Last-Modified", lastModified);
			} else {
				response.reset();
				response.setStatus(HttpServletResponse.SC_NOT_MODIFIED);
			}
		}
	}
}